<?php

/**
 *              
 *
 *                                                                           
 *  
 *                    
 *                      
 *                                                                   
 **/
function get_tag_end( $code, $start=0 )
{
$parser = new TagParser();
return $parser->getTagEnd($code, $start);
}    

class TagParser extends BaseObject {

var $bracket;
var $lt;
var $string;
var $parts;
var $isClosed;
var $lastTag;
var $lastPart;
var $simpleTags;

function TagParser($simpleTags = false) {
$this->simpleTags = $simpleTags;
$this->startTag();
}

function startTag() {
$this->bracket = 0;
$this->string = false;
$this->lt = 0;
$this->parts = array();
$this->isClosed = false;
$this->lastPart = "";
$this->lastTag = "";
}

function getTagEnd( $code, $start = 0, $addTags = false ) {
return $this->_parseTag($code, $start, $addTags);
}

function _parseTag($code, $start, $addTags) {
if ($this->simpleTags) {
return $this->_parseSimpleTag($code, $start, $addTags);
} else {
return $this->_parseCharTag($code, $start, $addTags);
}
}

function _parseSimpleTag($code, $start, $addTags) {
$pos = strpos($code, ">", $start);
if ($addTags) {
$this->lastPart .= ($pos === false)?substr($code, $start):substr($code, $start, $pos - $start);
if ($pos !== false) {
$this->_addPart($this->lastPart);
$this->lastPart = "";
}
}
return ($pos === false)?-1:$pos;
}

function _parseCharTag($code, $start, $addTags) {
$length = strlen( $code );
$stringOpener = "";
//                                                             
 $tagOpener = "";
for ( $i=$start; $i<$length; $i++ )
{
$ch = substr( $code, $i, 1 );
switch ( $ch )
{
case "\\":
$i++;
$ch = substr($code, $i, 1);
if ( $ch == "\\" ) break;
if ( !($this->string && $ch == $stringOpener) || $tagOpener == "(" )
if ( $addTags ) 
$this->lastPart .= "\\";
break;
case "(":
if ( !$tagOpener ) $tagOpener = "(";
if ( !$this->string )
$this->bracket++;
break;
case ")":
if ( !$this->string ) //                             
 $this->bracket--;
break;
case "<":
if ( !$this->string && !$this->bracket )
$this->lt++;
break;
case "'":
case "\"":
if ( !$tagOpener ) $tagOpener = "\""; //                                          
 if ( $this->string && $stringOpener != $ch )
break;
$this->string = !$this->string;
$stringOpener = $ch;
//                   
 if ($this->string) {
$next = strpos($code,$ch,$i);
if ($next === false) {
$ch = substr($code, $i);
$i = strlen($code);
} else if ($next > $i + 2) {
$ch = substr($code, $i, $next - $i - 1);
$i = $next - 1;
}
}
break;
case "/":
if (!$this->_inString()) { 
$this->isClosed = true;
$ch = "";
}
break;
case " ":
if ($addTags && !$this->_inString() && $this->lastPart) { 
$this->_addPart($this->lastPart);
$this->lastPart = "";
$ch = "";
}
break;
case ">":
if ( !$this->string )
{
if ( $this->lt > 0 ) {
$this->lt--;
} else if ( $this->bracket == 0 ) {
if ($addTags && $this->lastPart) { 
$this->_addPart($this->lastPart);
}
return $i;
}
}
break;
}
if (strlen($ch)>0 && $ch != " ") $this->isClosed = false;
//                          
 if ($addTags) $this->lastPart .= $ch;
}
if ($addTags && $this->lastPart && !$this->_inString()) {
$this->_addPart($this->lastPart);
$this->lastPart = "";
}
return -1;
}

function _inString() {
return ($this->string || $this->lt > 0 || $this->bracket > 0);
}

function parseTag($code, $start = 0) {
return $this->_parseTag($code, $start, true);
}

function isClosed() {
return $this->isClosed;
}

function getParts($hashed = true) {
$rs = array();
if ($hashed) {
reset($this->parts);
while (list($key,$val) = each($this->parts)) {
$rs[strtolower($val["key"])] = $val["val"];
}
} else {
$rs = $this->parts;
}
return $rs;
}

function _addPart($part) {
$pos = strpos($part, "=");
//                     
 $pos2 = strpos($part, "(");
if ($pos2 === 0 && count($this->parts) && $this->parts[count($this->parts)-1]["val"] === true) {
$this->parts[count($this->parts)-1]["val"] = $part;
return;
}
if ($pos2 > 0 && ($pos === false || $pos2 < $pos)) {
$pos = $pos2;
$key = substr($part, 0, $pos);
$val = substr($part, $pos);
} else if ($pos !== false) {
$key = substr($part, 0, $pos);
$val = substr($part, $pos + 1);
}

if ($pos === false) {
$this->parts[] = array("key"=>$part, "val"=>true);
} else {
if (substr($val, 0, 1) == "\"" && substr($val, -1) == "\"") {
$val = substr($val, 1, -1);
}
$this->parts[] = array("key"=>$key, "val" => $val);
}
}
}

?>
